@@ -1,16 +1,22 @@
 #
 # SNMP Devices Simulator
 #
-# Written by Ilya Etingof <ilya@glas.net>, 05/2010
+# Written by Ilya Etingof <ilya@glas.net>, 2010-2011
 #
 import os
 import sys
-import string
 import getopt
-import hashlib
+if sys.version_info[0] < 3 and sys.version_info[1] < 5:
+    from md5 import md5
+else:
+    from hashlib import md5
 import time
-import anydbm
-import whichdb
+if sys.version_info[0] < 3:
+    import anydbm as dbm
+    from whichdb import whichdb
+else:
+    import dbm
+    whichdb = dbm.whichdb
 import bisect
 from pyasn1.type import univ
 from pyasn1.codec.ber import encoder, decoder
@@ -43,18 +49,17 @@ try:
     opts, params = getopt.getopt(sys.argv[1:], 'h',
         ['help', 'debug=', 'device-dir=', 'force-index-rebuild', 'validate-device-data', 'agent-address=', 'agent-port=', 'v2c-arch', 'v3-only', 'v3-user=', 'v3-auth-key=', 'v3-priv-key=', 'v3-priv-proto=']
         )
-except Exception, why:
-    print why
-    print helpMessage
+except Exception:
+    sys.stdout.write('%s\r\n%s\r\n' % (sys.exc_info()[1], helpMessage))
     sys.exit(-1)
 
 if params:
-    print 'extra arguments supplied %s\n' % (params) + helpMessage
+    sys.stdout.write('extra arguments supplied %s%s\r\n' % (params, helpMessage))
     sys.exit(-1)
 
 for opt in opts:
     if opt[0] == '-h' or opt[0] == '--help':
-        print helpMessage
+        sys.stdout.write('%s\r\n' % helpMessage)
         sys.exit(-1)
     elif opt[0] == '--debug':
         debug.setLogger(debug.Debug(opt[1]))
@@ -67,7 +72,7 @@ for opt in opts:
     elif opt[0] == '--agent-address':
         agentAddress = (opt[1], agentAddress[1])
     elif opt[0] == '--agent-port':
-        agentAddress = (agentAddress[0], string.atoi(opt[1]))
+        agentAddress = (agentAddress[0], int(opt[1]))
     elif opt[0] == '--v2c-arch':
         v2cArch = True
     elif opt[0] == '--v3-only':
@@ -84,7 +89,7 @@ for opt in opts:
         elif opt[1].upper() == 'AES':
             v3PrivProto = config.usmAesCfb128Protocol
         else:
-            print 'bad v3 privacy protocol'
+            sys.stdout.write('bad v3 privacy protocol\r\n')
             sys.exit(-1)
 
 # Device file entry parsers
@@ -105,12 +110,11 @@ class DumpParser:
         }
 
     def __nullFilter(value):
-        return "" # simply drop whatever value is there when it's a Null
+        return '' # simply drop whatever value is there when it's a Null
     
     def __unhexFilter(value):
-        if value[:5].lower() == "hex: ":
-            value = ''.join(map(chr, map(lambda x: string.atoi(x, 16),
-                                     value[5:].split('.'))))
+        if value[:5].lower() == 'hex: ':
+            value = [ int(x, 16) for x in value[5:].split('.') ]
         elif value[0] == '"' and value[-1] == '"':
             value = value[1:-1]
         return value
@@ -162,7 +166,7 @@ class SnmprecParser:
                rfc1902.Opaque,
                rfc1902.Counter32,
                rfc1902.Counter64 ):
-        tagMap[str(reduce(lambda x,y: x+y, t.tagSet[0]))] = t
+        tagMap[str(sum([ x for x in t.tagSet[0] ]))] = t
 
     def parse(self, line): return line.strip().split('|', 2)
 
@@ -173,10 +177,7 @@ class SnmprecParser:
         # Unhexify
         if tag[-1] == 'x':
             tag = tag[:-1]
-            value = ''.join(
-                map(lambda x: chr(int(value[x:x+2], 16)),
-                    range(0, len(value), 2))
-                )
+            value = [int(value[x:x+2], 16) for x in  range(0, len(value), 2)]
         return tag, self.tagMap[tag](value)
 
     def evaluate(self, line, oidOnly=False, muteValueError=True):
@@ -212,8 +213,8 @@ class DeviceFile:
             self.__dbFile = textFile[:textFile.rindex(os.path.extsep)]
         except ValueError:
             self.__dbFile = textFile
-        finally:
-            self.__dbFile = self.__dbFile + os.path.extsep + 'dbm'
+
+        self.__dbFile = self.__dbFile + os.path.extsep + 'dbm'
     
         self.__db = self.__text = None
         self.__dbType = '?'
@@ -232,21 +233,22 @@ class DeviceFile:
             if os.path.exists(dbFile):
                 if textFileStamp < os.stat(dbFile)[8]:
                     if indexNeeded:
-                        print "Forced index rebuild %s" % dbFile
+                        sys.stdout.write('Forced index rebuild %s\r\n' % dbFile)
                 else:
                     indexNeeded = True
-                    print "Index %s out of date" % dbFile
+                    sys.stdout.write('Index %s out of date\r\n' % dbFile)
                 break
         else:
             indexNeeded = True
-            print "Index does not exist for %s" % self.__textFile
+            sys.stdout.write('Index does not exist for %s\r\n' % self.__textFile)
             
         if indexNeeded:
-            print "Indexing device file %s..." % self.__textFile
+            sys.stdout.write('Indexing device file %s...' % self.__textFile)
+            sys.stdout.flush()
 
             text = open(self.__textFile)
             
-            db = anydbm.open(self.__dbFile, 'n')
+            db = dbm.open(self.__dbFile, 'n')
         
             lineNo = 0
             offset = 0
@@ -259,31 +261,32 @@ class DeviceFile:
 
                 try:
                     oid, tag, val = textParser.parse(line)
-                except Exception, why:
+                except Exception:
                     db.close()
                     os.remove(self.__dbFile)
                     raise Exception(
                         'Data error at %s:%d: %s' % (
-                            self.__textFile, lineNo, why
+                            self.__textFile, lineNo, sys.exc_info()[1]
                             )
                         )
 
                 if validateData:
                     try:
                         textParser.evaluateOid(oid)
-                    except Exception, why:
+                    except Exception:
                         db.close()
                         os.remove(self.__dbFile)
                         raise Exception(
                             'OID error at %s:%d: %s' % (
-                                self.__textFile, lineNo, why
+                                self.__textFile, lineNo, sys.exc_info()[1]
                                 )
                             )
                     try:
                         textParser.evaluateValue(tag, val)
-                    except Exception, why:
-                        print '*** Error at line %s, value %s: %s' % (
-                            lineNo, repr(val), why
+                    except Exception:
+                        sys.stdout.write(
+                            '*** Error at line %s, value %r: %s\r\n' % \
+                            (lineNo, val, sys.exc_info()[1])
                             )
                         
                 db[oid] = str(offset)
@@ -293,9 +296,9 @@ class DeviceFile:
             text.close()
             db.close()
         
-            print "...%d entries indexed" % (lineNo - 1,)
+            sys.stdout.write('...%d entries indexed\r\n' % (lineNo - 1,))
 
-        self.__dbType = whichdb.whichdb(self.__dbFile)
+        self.__dbType = whichdb(self.__dbFile)
 
         return self
 
@@ -313,14 +316,14 @@ class DeviceFile:
             DeviceFile.openedQueue.append(self)
 
             self.__text = open(self.__textFile)
-
-            self.__db = anydbm.open(self.__dbFile)
+            
+            self.__db = dbm.open(self.__dbFile)
 
         return self.__text, self.__db
 
     def __str__(self):
         return 'file %s, %s-indexed, %s' % (
-            self.__textFile, self.__dbType, self.__db and "opened" or "closed"
+            self.__textFile, self.__dbType, self.__db and 'opened' or 'closed'
             )
 
 # Collect device files
@@ -341,7 +344,7 @@ def getDevices(tgtDir, topLen=None):
             dExt = dFile[dFile.rindex(os.path.extsep):]
         except ValueError:
             continue
-        if not parserSet.has_key(dExt):
+        if dExt not in parserSet:
             continue
         fullPath = tgtDir + os.path.sep + dFile
         relPath = fullPath.split(os.path.sep)[topLen:]
@@ -379,9 +382,9 @@ class MibInstrumController:
             line = text.readline()
             try:
                 midval, _ = self.__textParser.evaluate(line, oidOnly=True)
-            except Exception, why:
+            except Exception:
                 raise Exception(
-                    'Data error at %s for %s: %s' % (self, oid, why)
+                    'Data error at %s for %s: %s' % (self,oid,sys.exc_info()[1])
                     )                
             if midval < oid:
                 lo = mid + len(line)
@@ -407,11 +410,13 @@ class MibInstrumController:
         text, db = self.__deviceFile.getHandles()
         
         for oid, val in varBinds:
-            textOid = reduce(lambda x,y: '%s.%s' % (x, y), oid)
-            if db.has_key(textOid):
+            textOid = univ.OctetString(
+                '.'.join([ '%s' % x for x in oid ])
+                ).asOctets()
+            try:
                 offset = db[textOid]
                 exactMatch = True
-            else:
+            except KeyError:
                 if nextFlag:
                     offset = self.__searchOid(text, oid)
                     exactMatch = False
@@ -419,7 +424,7 @@ class MibInstrumController:
                     rspVarBinds.append((oid, errorStatus))
                     continue
 
-            offset = long(offset)
+            offset = int(offset)
 
             text.seek(offset)
 
@@ -434,22 +439,22 @@ class MibInstrumController:
 
             try:
                 oid, val = self.__textParser.evaluate(line)
-            except Exception, why:
+            except Exception:
                 raise Exception(
-                    'Data error at %s for %s: %s' % (self, textOid, why)
+                    'Data error at %s for %s: %s' % (self, textOid, sys.exc_info()[1])
                     )
 
             rspVarBinds.append((oid, val))
 
         return rspVarBinds
     
-    def readVars(self, varBinds, (acFun, acCtx)=(None, None)):
+    def readVars(self, varBinds, acInfo=None):
         return self.__doVars(varBinds, False)
 
-    def readNextVars(self, varBinds, (acFun, acCtx)=(None, None)):
+    def readNextVars(self, varBinds, acInfo=None):
         return self.__doVars(varBinds, True)
 
-    def writeVars(self, varBinds, (acFun, acCtx)=(None, None)):
+    def writeVars(self, varBinds, acInfo=None):
         return self.__doVars(varBinds, False, True)
 
 # Devices index as a MIB instrumentaion at a dedicated SNMP context
@@ -461,11 +466,8 @@ class DevicesIndexInstrumController:
         self.__indexOid = baseOid + self.indexSubOid
         self.__idx = 1
 
-    def readVars(self, varBinds, (acFun, acCtx)=(None, None)):
-        return map(
-            lambda (o,v): (o, self.__db.get(o, exval.noSuchInstance)),
-            varBinds
-            )
+    def readVars(self, varBinds, acInfo=None):
+        return [ (vb[0], self.__db.get(vb[0], exval.noSuchInstance)) for vb in varBinds ]
 
     def __getNextVal(self, key, default):
         try:
@@ -475,15 +477,11 @@ class DevicesIndexInstrumController:
         else:
             return key, self.__db[key]
                                                             
-    def readNextVars(self, varBinds, (acFun, acCtx)=(None, None)):
-        return map(
-            lambda (o,v): self.__getNextVal(o, exval.endOfMib), varBinds
-            )
+    def readNextVars(self, varBinds, acInfo=None):
+        return [ (vb[0], self.__getNextVal(vb[0], exval.endOfMib)) for vb in varBinds ]        
 
-    def writeVars(self, varBinds, (acFun, acCtx)=(None, None)):
-        return map(
-            lambda (o,v): (o, exval.noSuchInstance), varBinds
-            )
+    def writeVars(self, varBinds, acInfo=None):
+        return [ (vb[0], exval.noSuchInstance) for vb in varBinds ]        
     
     def addDevice(self, *args):
         for idx in range(len(args)):
@@ -518,31 +516,32 @@ for fullPath, textParser, communityName in getDevices(deviceDir):
         DeviceFile(fullPath).indexText(textParser, forceIndexBuild), textParser
         )
 
-    print """Device %s
-SNMPv1/2c community name: %s""" % (mibInstrum, communityName)
+    sys.stdout.write('Device %s\r\nSNMPv1/2c community name: %s\r\n' % \
+                     (mibInstrum, communityName))
 
     if v2cArch:
-        contexts[communityName] = mibInstrum
+        contexts[univ.OctetString(communityName)] = mibInstrum
         
         devicesIndexInstrumController.addDevice(
             fullPath, communityName
             )
     else:
-        agentName = contextName = hashlib.md5(communityName).hexdigest()
+        agentName = contextName = md5(univ.OctetString(communityName).asOctets()).hexdigest()
 
         if not v3Only:
             config.addV1System(
                 snmpEngine, agentName, communityName, contextName=contextName
                 )
-        
+
         snmpContext.registerContextName(contextName, mibInstrum)
                  
         devicesIndexInstrumController.addDevice(
             fullPath, communityName, contextName
             )
                  
-        print """SNMPv3 context name: %s
-""" % (contextName,)
+        sys.stdout.write('SNMPv3 context name: %s\r\n' % (contextName,))
+        
+    sys.stdout.write('%s\r\n' % ('-+' * 33,))
         
 if v2cArch:
     def getBulkHandler(varBinds, nonRepeaters, maxRepetitions, readNextVars):
@@ -568,20 +567,20 @@ if v2cArch:
                               transportAddress, wholeMsg):
         while wholeMsg:
             msgVer = api.decodeMessageVersion(wholeMsg)
-            if api.protoModules.has_key(msgVer):
+            if msgVer in api.protoModules:
                 pMod = api.protoModules[msgVer]
             else:
-                print "Unsupported SNMP version %s" % msgVer
+                sys.stdout.write('Unsupported SNMP version %s\r\n' % (msgVer,))
                 return
             reqMsg, wholeMsg = decoder.decode(
                 wholeMsg, asn1Spec=pMod.Message(),
                 )
             
             communityName = reqMsg.getComponentByPosition(1)
-            if not contexts.has_key(communityName):
-                print "Unknown community name %s from %s:%s\n" % (
+            if communityName not in contexts:
+                sys.stdout.write('Unknown community name %s from %s:%s\r\n' % (
                     communityName, transportDomain, transportAddress
-                    )
+                    ))
                 return wholeMsg
             
             rspMsg = pMod.apiMessage.getResponse(reqMsg)
@@ -597,19 +596,19 @@ if v2cArch:
             elif hasattr(pMod, 'GetBulkRequestPDU') and \
                      reqPDU.isSameTypeWith(pMod.GetBulkRequestPDU()):
                 if not msgVer:
-                    print "GETBULK over SNMPv1 from %s:%s\n" % (
+                    sys.stdout.write('GETBULK over SNMPv1 from %s:%s\r\n' % (
                         transportDomain, transportAddress
-                        )
+                        ))
                     return wholeMsg
                 backendFun = lambda varBinds: getBulkHandler(varBinds,
                     pMod.apiBulkPDU.getNonRepeaters(reqPDU),
                     pMod.apiBulkPDU.getMaxRepetitions(reqPDU),
                     contexts[communityName].readNextVars)
             else:
-                print "Unsuppored PDU type %s from %s:%s\n" % (
+                sys.stdout.write('Unsuppored PDU type %s from %s:%s\r\n' % (
                     reqPDU.__class__.__name__, transportDomain,
                     transportAddress
-                    )
+                    ))
                 return wholeMsg
     
             varBinds = backendFun(
@@ -648,11 +647,7 @@ if v2cArch:
             )
     transportDispatcher.registerRecvCbFun(commandResponderCbFun)
 else:
-    print """SNMPv3 credentials:
-Username: %s
-Authentication key: %s
-Encryption (privacy) key: %s
-Encryption protocol: %s""" % (v3User, v3AuthKey, v3PrivKey, v3PrivProto)
+    sys.stdout.write('SNMPv3 credentials:\r\nUsername: %s\r\nAuthentication key: %s\r\nEncryption (privacy) key: %s\r\nEncryption protocol: %s\r\n' % (v3User, v3AuthKey, v3PrivKey, v3PrivProto))
 
     # Configure access to devices index
 
@@ -680,9 +675,7 @@ Encryption protocol: %s""" % (v3User, v3AuthKey, v3PrivKey, v3PrivProto)
 
     transportDispatcher = snmpEngine.transportDispatcher
     
-print """
-Listening at %s
-""" % (agentAddress,)
+sys.stdout.write('\r\nListening at %s\r\n' % (agentAddress,))
 
 # Run mainloop
 
